=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2009-02-24 06:55:13 +0000
+++ bzrlib/builtins.py	2009-02-25 13:25:14 +0000
@@ -2304,6 +2304,14 @@
         if revision is not None or tree is None:
             tree = _get_one_revision_tree('ls', revision, branch=branch)
 
+        apply_view = False
+        if tree.supports_views():
+            view_files = tree.views.lookup_view()
+            if view_files:
+                apply_view = True
+                view_str = views.view_display_str(view_files)
+                note("ignoring files outside view: %s" % view_str)
+
         tree.lock_read()
         try:
             for fp, fc, fkind, fid, entry in tree.list_files(include_root=False):
@@ -2315,6 +2323,11 @@
                         continue
                     if kind is not None and fkind != kind:
                         continue
+                    if apply_view:
+                        try:
+                            views.check_path_in_view(tree, fp)
+                        except errors.FileOutsideView:
+                            continue
                     kindch = entry.kind_character()
                     outstring = fp + kindch
                     if verbose:

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2009-02-23 15:29:35 +0000
+++ bzrlib/diff.py	2009-02-25 13:25:14 +0000
@@ -334,7 +334,7 @@
         bzrdir.BzrDir.open_containing_tree_or_branch(old_url)
     if consider_relpath and relpath != '':
         if working_tree is not None and apply_view:
-            _check_path_in_view(working_tree, relpath)
+            views.check_path_in_view(working_tree, relpath)
         specific_files.append(relpath)
     old_tree = _get_tree_to_diff(old_revision_spec, working_tree, branch)
 
@@ -346,7 +346,7 @@
             bzrdir.BzrDir.open_containing_tree_or_branch(new_url)
         if consider_relpath and relpath != '':
             if working_tree is not None and apply_view:
-                _check_path_in_view(working_tree, relpath)
+                views.check_path_in_view(working_tree, relpath)
             specific_files.append(relpath)
     new_tree = _get_tree_to_diff(new_revision_spec, working_tree, branch,
         basis_is_default=working_tree is None)
@@ -376,15 +376,6 @@
         extra_trees = (working_tree,)
     return old_tree, new_tree, specific_files, extra_trees
 
-
-def _check_path_in_view(tree, relpath):
-    """If a working tree has a view enabled, check the path is within it."""
-    if tree.supports_views():
-        view_files = tree.views.lookup_view()
-        if  view_files and not osutils.is_inside_any(view_files, relpath):
-            raise errors.FileOutsideView(relpath, view_files)
-
-
 def _get_tree_to_diff(spec, tree=None, branch=None, basis_is_default=True):
     if branch is None and tree is not None:
         branch = tree.branch

=== modified file 'bzrlib/tests/blackbox/test_filtered_view_ops.py'
--- bzrlib/tests/blackbox/test_filtered_view_ops.py	2009-02-11 00:41:15 +0000
+++ bzrlib/tests/blackbox/test_filtered_view_ops.py	2009-02-24 22:56:13 +0000
@@ -142,6 +142,15 @@
                           'current view: a, b\n', err)
         self.assertEquals('', out)
 
+    def test_view_on_ls(self):
+        wt = self.make_abc_tree_with_ab_view()
+        self.run_bzr('add')
+        out, err = self.run_bzr('ls')
+        out_lines = out.splitlines()
+        self.assertEquals('ignoring files outside view: a, b\n', err)
+        self.assertEquals('a', out_lines[0])
+        self.assertEquals('b', out_lines[1])
+
 
 class TestViewTreeOperationss(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/workingtree_implementations/test_views.py'
--- bzrlib/tests/workingtree_implementations/test_views.py	2009-02-23 15:42:47 +0000
+++ bzrlib/tests/workingtree_implementations/test_views.py	2009-02-25 13:25:14 +0000
@@ -21,7 +21,7 @@
 """
 
 
-from bzrlib import errors
+from bzrlib import views, errors
 from bzrlib.tests import TestSkipped
 from bzrlib.workingtree import WorkingTree
 
@@ -141,6 +141,22 @@
         self.assertRaises(errors.NoSuchView,
             wt.views.delete_view, view_name + '2')
 
+    def test_check_path_in_view(self):
+        wt = self.make_branch_and_tree('wt')
+        view_current = 'view-name'
+        view_dict = {
+            view_current: ['dir-1'],
+            'other-name': ['dir-2']}
+        wt.views.set_view_info(view_current, view_dict)
+        self.assertEqual(views.check_path_in_view(wt, 'dir-1'), None)
+        self.assertEqual(views.check_path_in_view(wt, 'dir-1/sub'), None)
+        self.assertRaises(errors.FileOutsideView,
+                          views.check_path_in_view, wt, 'dir-2')
+        self.assertRaises(errors.FileOutsideView,
+                          views.check_path_in_view, wt, 'dir-2/sub')
+        self.assertRaises(errors.FileOutsideView,
+                          views.check_path_in_view, wt, 'other')
+
 
 class TestUnsupportedViews(TestCaseWithWorkingTree):
     """Formats that don't support views should give reasonable errors."""

=== modified file 'bzrlib/views.py'
--- bzrlib/views.py	2009-02-23 15:42:47 +0000
+++ bzrlib/views.py	2009-02-25 13:25:14 +0000
@@ -27,6 +27,7 @@
 
 from bzrlib import (
     errors,
+    osutils,
     )
 
 
@@ -273,3 +274,11 @@
         return ", ".join(view_files)
     else:
         return ", ".join([v.encode(encoding, 'replace') for v in view_files])
+
+
+def check_path_in_view(tree, relpath):
+    """If a working tree has a view enabled, check the path is within it."""
+    if tree.supports_views():
+        view_files = tree.views.lookup_view()
+        if  view_files and not osutils.is_inside_any(view_files, relpath):
+            raise errors.FileOutsideView(relpath, view_files)

